var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var __decorate = this && this.__decorate || function (decorators, target, key, desc) {
    var c = arguments.length,
        r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc,
        d;
    if ((typeof Reflect === "undefined" ? "undefined" : _typeof(Reflect)) === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);else for (var i = decorators.length - 1; i >= 0; i--) {
        if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    }return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = this && this.__metadata || function (k, v) {
    if ((typeof Reflect === "undefined" ? "undefined" : _typeof(Reflect)) === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = this && this.__param || function (paramIndex, decorator) {
    return function (target, key) {
        decorator(target, key, paramIndex);
    };
};
import { ElementRef, Component, Directive, HostBinding, ViewChild, Input, Output, EventEmitter, Optional, ViewEncapsulation } from '@angular/core';
import { NgControl } from '@angular/common';
import { Config } from '../../config/config';
import { Ion } from '../ion';
import { isPresent } from '../../util/util';
/**
* @private
*/
export var SearchbarInput = function SearchbarInput(elementRef) {
    _classCallCheck(this, SearchbarInput);

    this.elementRef = elementRef;
};
SearchbarInput = __decorate([Directive({
    selector: '.searchbar-input'
}), __metadata('design:paramtypes', [typeof (_a = typeof ElementRef !== 'undefined' && ElementRef) === 'function' && _a || Object])], SearchbarInput);
/**
 * @name Searchbar
 * @module ionic
 * @description
 * Manages the display of a Searchbar which can be used to search or filter items.
 *
 * @usage
 * ```html
 * <ion-searchbar
 *   [(ngModel)]="myInput"
 *   [hideCancelButton]="shouldHideCancel"
 *   (ionInput)="onInput($event)"
 *   (ionCancel)="onCancel($event)">
 * </ion-searchbar>
 * ```
 *
 * @demo /docs/v2/demos/searchbar/
 * @see {@link /docs/v2/components#searchbar Searchbar Component Docs}
 */
export var Searchbar = function (_Ion) {
    _inherits(Searchbar, _Ion);

    function Searchbar(_elementRef, _config, ngControl) {
        _classCallCheck(this, Searchbar);

        var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(Searchbar).call(this, _elementRef));

        _this._elementRef = _elementRef;
        _this._config = _config;
        /**
         * @input {number} How long, in milliseconds, to wait to trigger the `input` event after each keystroke. Default `250`.
         */
        _this.debounce = 250;
        /**
         * @output {event} When the Searchbar input has changed including cleared
         */
        _this.ionInput = new EventEmitter();
        /**
         * @output {event} When the Searchbar input has blurred
         */
        _this.ionBlur = new EventEmitter();
        /**
         * @output {event} When the Searchbar input has focused
         */
        _this.ionFocus = new EventEmitter();
        /**
         * @output {event} When the cancel button is clicked
         */
        _this.ionCancel = new EventEmitter();
        /**
         * @output {event} When the clear input button is clicked
         */
        _this.ionClear = new EventEmitter();
        /**
         * @private
         */
        _this.value = '';
        /**
         * @private
         */
        _this.blurInput = true;
        /**
         * @private
         */
        _this.onChange = function (_) {};
        /**
         * @private
         */
        _this.onTouched = function () {};
        // If the user passed a ngControl we need to set the valueAccessor
        if (ngControl) {
            ngControl.valueAccessor = _this;
        }
        return _this;
    }
    /**
     * @private
     * On Initialization check for attributes
     */


    _createClass(Searchbar, [{
        key: "ngOnInit",
        value: function ngOnInit() {
            this.mode = this._config.get('mode');
            var hideCancelButton = this.hideCancelButton;
            if (typeof hideCancelButton === 'string') {
                this.hideCancelButton = hideCancelButton === '' || hideCancelButton === 'true';
            }
            this.cancelButtonText = this.cancelButtonText || 'Cancel';
            this.placeholder = this.placeholder || 'Search';
            if (this.ngModel) this.value = this.ngModel;
            this.onChange(this.value);
            this.shouldLeftAlign = this.value && this.value.trim() !== '';
            // Using querySelector instead of searchbarInput because at this point it doesn't exist
            this.inputElement = this._elementRef.nativeElement.querySelector('.searchbar-input');
            this.searchIconElement = this._elementRef.nativeElement.querySelector('.searchbar-search-icon');
            this.setElementLeft();
        }
        /**
         * @private
         * After View Initialization check the value
         */

    }, {
        key: "ngAfterViewInit",
        value: function ngAfterViewInit() {
            // If the user passes an undefined variable to ngModel this will warn
            // and set the value to an empty string
            if (!isPresent(this.value)) {
                console.warn('Searchbar was passed an undefined value in ngModel. Please make sure the variable is defined.');
                this.value = '';
                this.onChange(this.value);
            }
        }
        /**
         * @private
         * Determines whether or not to add style to the element
         * to center it properly (ios only)
         */

    }, {
        key: "setElementLeft",
        value: function setElementLeft() {
            if (this.mode !== 'ios') return;
            if (this.shouldLeftAlign) {
                this.inputElement.removeAttribute('style');
                this.searchIconElement.removeAttribute('style');
            } else {
                this.addElementLeft();
            }
        }
        /**
         * @private
         * Calculates the amount of padding/margin left for the elements
         * in order to center them based on the placeholder width
         */

    }, {
        key: "addElementLeft",
        value: function addElementLeft() {
            // Create a dummy span to get the placeholder width
            var tempSpan = document.createElement('span');
            tempSpan.innerHTML = this.placeholder;
            document.body.appendChild(tempSpan);
            // Get the width of the span then remove it
            var textWidth = tempSpan.offsetWidth;
            tempSpan.remove();
            // Set the input padding left
            var inputLeft = 'calc(50% - ' + textWidth / 2 + 'px)';
            this.inputElement.style.paddingLeft = inputLeft;
            // Set the icon margin left
            var iconLeft = 'calc(50% - ' + (textWidth / 2 + 30) + 'px)';
            this.searchIconElement.style.marginLeft = iconLeft;
        }
        /**
         * @private
         * Update the Searchbar input value when the input changes
         */

    }, {
        key: "inputChanged",
        value: function inputChanged(ev) {
            var _this2 = this;

            var value = ev.target.value;
            clearTimeout(this._tmr);
            this._tmr = setTimeout(function () {
                _this2.value = value;
                _this2.onChange(value);
                _this2.ionInput.emit(_this2);
            }, Math.round(this.debounce));
        }
        /**
         * @private
         * Sets the Searchbar to focused and aligned left on input focus.
         */

    }, {
        key: "inputFocused",
        value: function inputFocused() {
            this.ionFocus.emit(this);
            this.isFocused = true;
            this.shouldLeftAlign = true;
            this.setElementLeft();
        }
        /**
         * @private
         * Sets the Searchbar to not focused and checks if it should align left
         * based on whether there is a value in the searchbar or not.
         */

    }, {
        key: "inputBlurred",
        value: function inputBlurred() {
            // blurInput determines if it should blur
            // if we are clearing the input we still want to stay focused in the input
            if (this.blurInput === false) {
                this.searchbarInput.elementRef.nativeElement.focus();
                this.blurInput = true;
                return;
            }
            this.ionBlur.emit(this);
            this.isFocused = false;
            this.shouldLeftAlign = this.value && this.value.trim() !== '';
            this.setElementLeft();
        }
        /**
         * @private
         * Clears the input field and triggers the control change.
         */

    }, {
        key: "clearInput",
        value: function clearInput() {
            this.ionClear.emit(this);
            this.value = '';
            this.onChange(this.value);
            this.ionInput.emit(this);
            this.blurInput = false;
        }
        /**
         * @private
         * Clears the input field and tells the input to blur since
         * the clearInput function doesn't want the input to blur
         * then calls the custom cancel function if the user passed one in.
         */

    }, {
        key: "cancelSearchbar",
        value: function cancelSearchbar() {
            this.ionCancel.emit(this);
            this.clearInput();
            this.blurInput = true;
        }
        /**
         * @private
         * Write a new value to the element.
         */

    }, {
        key: "writeValue",
        value: function writeValue(value) {
            this.value = value;
        }
        /**
         * @private
         * Set the function to be called when the control receives a change event.
         */

    }, {
        key: "registerOnChange",
        value: function registerOnChange(fn) {
            this.onChange = fn;
        }
        /**
         * @private
         * Set the function to be called when the control receives a touch event.
         */

    }, {
        key: "registerOnTouched",
        value: function registerOnTouched(fn) {
            this.onTouched = fn;
        }
    }]);

    return Searchbar;
}(Ion);
__decorate([ViewChild(SearchbarInput), __metadata('design:type', SearchbarInput)], Searchbar.prototype, "searchbarInput", void 0);
__decorate([Input(), __metadata('design:type', String)], Searchbar.prototype, "cancelButtonText", void 0);
__decorate([Input(), __metadata('design:type', Object)], Searchbar.prototype, "hideCancelButton", void 0);
__decorate([Input(), __metadata('design:type', Number)], Searchbar.prototype, "debounce", void 0);
__decorate([Input(), __metadata('design:type', String)], Searchbar.prototype, "placeholder", void 0);
__decorate([Input(), __metadata('design:type', Object)], Searchbar.prototype, "ngModel", void 0);
__decorate([Output(), __metadata('design:type', typeof (_b = typeof EventEmitter !== 'undefined' && EventEmitter) === 'function' && _b || Object)], Searchbar.prototype, "ionInput", void 0);
__decorate([Output(), __metadata('design:type', typeof (_c = typeof EventEmitter !== 'undefined' && EventEmitter) === 'function' && _c || Object)], Searchbar.prototype, "ionBlur", void 0);
__decorate([Output(), __metadata('design:type', typeof (_d = typeof EventEmitter !== 'undefined' && EventEmitter) === 'function' && _d || Object)], Searchbar.prototype, "ionFocus", void 0);
__decorate([Output(), __metadata('design:type', typeof (_e = typeof EventEmitter !== 'undefined' && EventEmitter) === 'function' && _e || Object)], Searchbar.prototype, "ionCancel", void 0);
__decorate([Output(), __metadata('design:type', typeof (_f = typeof EventEmitter !== 'undefined' && EventEmitter) === 'function' && _f || Object)], Searchbar.prototype, "ionClear", void 0);
__decorate([HostBinding('class.searchbar-focused'), __metadata('design:type', Boolean)], Searchbar.prototype, "isFocused", void 0);
__decorate([HostBinding('class.searchbar-left-aligned'), __metadata('design:type', Boolean)], Searchbar.prototype, "shouldLeftAlign", void 0);
Searchbar = __decorate([Component({
    selector: 'ion-searchbar',
    host: {
        '[class.searchbar-has-value]': 'value',
        '[class.searchbar-hide-cancel]': 'hideCancelButton'
    },
    template: '<div class="searchbar-input-container">' + '<button (click)="cancelSearchbar()" (mousedown)="cancelSearchbar()" [hidden]="hideCancelButton" clear dark class="searchbar-md-cancel">' + '<ion-icon name="arrow-back"></ion-icon>' + '</button>' + '<div class="searchbar-search-icon"></div>' + '<input [value]="value" (input)="inputChanged($event)" (blur)="inputBlurred()" (focus)="inputFocused()" class="searchbar-input" type="search" [attr.placeholder]="placeholder" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false">' + '<button clear class="searchbar-clear-icon" (click)="clearInput()" (mousedown)="clearInput()"></button>' + '</div>' + '<button clear (click)="cancelSearchbar()" (mousedown)="cancelSearchbar()" [hidden]="hideCancelButton" class="searchbar-ios-cancel">{{cancelButtonText}}</button>',
    directives: [SearchbarInput],
    encapsulation: ViewEncapsulation.None
}), __param(2, Optional()), __metadata('design:paramtypes', [typeof (_g = typeof ElementRef !== 'undefined' && ElementRef) === 'function' && _g || Object, typeof (_h = typeof Config !== 'undefined' && Config) === 'function' && _h || Object, typeof (_j = typeof NgControl !== 'undefined' && NgControl) === 'function' && _j || Object])], Searchbar);
var _a, _b, _c, _d, _e, _f, _g, _h, _j;